C***********************************************************************BEGIN-HDR
C 
C  Copyright  1997-2002 Reaction Design.  All rights reserved.
C
C  $Header: /chemkin/unix/user_routines/aurora_user_routines.f 14    2/12/03 10:53a Frupley $
C
C  $NoKeywords: $ 
C***********************************************************************END-HDR
C
C //////////////   USER ROUTINES ///////////////////////////////////
C
C-----------------------------------------------------------------------
C
      SUBROUTINE PSTEMPT (TIME, LOUT, T, PA)
C  This is a USER SUBROUTINE for defining Temperature
C  and Pressure as an arbitrary function of time.
C  Use of this subroutine is controlled by the AURORA
C  keyword, TTIM, as described in the AURORA user's manual.
C
C  The subroutine PSTEMPT must return values of
C     T   - Temperature in Kelvin
C     PA  - pressure in atmospheres
C  Given the values of
C    TIME - time in seconds.
C    LOUT - if positive, the unit number of an open output file
C
C*****EXCLUDE-IF precision_single
      IMPLICIT DOUBLE PRECISION (A-H, O-Z), INTEGER (I-N)
C*****END EXCLUDE-IF precision_single
C*****INCLUDE-IF precision_single
C      IMPLICIT REAL (A-H, O-Z), INTEGER (I-N)
C*****END INCLUDE-IF precision_single
      RETURN
      END
C
C-----------------------------------------------------------------------
C
      SUBROUTINE PSVOLT (TIME, LOUT, VOL, DVDT)
C  This is a USER SUBROUTINE for defining System Volume
C  as an arbitrary function of time.
C  Use of this subroutine is controlled by the AURORA
C  keyword, VTIM, as described in the AURORA user's manual.
C
C  The subroutine PSVOLT must return values of
C      VOL  - volume of the system (cm**3)
C      DVDT - time derivative of the volume with respect to time
C             (cm**3/s)
C  Given
C      TIME - time in seconds.
C      LOUT - if positive, the unit number of an open output file
C
C  Note: Consistent units for volume are (cm3), but the volume can
C        be considered to be normalized and therefore unitless.
C        The problem is solved using intensive variables, so the
C        solution is independent of extensive variables such as
C        the volume and total mass.  Subroutine VOLT is called at
C        time zero and a total mass is computed, but the solution
C        only depends on the density, so the total mass and total
C        volume are not important.
C
C*****EXCLUDE-IF precision_single
      IMPLICIT DOUBLE PRECISION (A-H, O-Z), INTEGER (I-N)
C*****END EXCLUDE-IF precision_single
C*****INCLUDE-IF precision_single
C      IMPLICIT REAL (A-H, O-Z), INTEGER (I-N)
C*****END INCLUDE-IF precision_single
C
      RETURN
      END
C
C-----------------------------------------------------------------------
C
C*****EXCLUDE-IF precision_single
      DOUBLE PRECISION FUNCTION PSQFUN (LENIQ, LENRQ, IQFUN, RQFUN)
      IMPLICIT DOUBLE PRECISION (A-H, O-Z), INTEGER (I-N)
C*****END EXCLUDE-IF precision_single
C*****INCLUDE-IF precision_single
C      REAL FUNCTION PSQFUN (LENIQ, LENRQ, IQFUN, RQFUN)
C      IMPLICIT REAL (A-H, O-Z), INTEGER (I-N)
C*****END INCLUDE-IF precision_single
      DIMENSION IQFUN(LENIQ), RQFUN(LENRQ)
C
C This function is used to specify a time-dependent heat loss (erg/sec) 
C to the outside reactor wall if the keyword 'QFUN' appears in an input 
C file.  Aurora provides some current solution parameters to the user 
C in the integer array IQFUN and the real array RQFUN.
C !!Please ensure that the values of LENIQ, LENRQ do not change!!
C
C  Formatted output may be printed to
C    LOUT - if positive, the unit number of an open output file
      LOUT = IQFUN(3)
C
C     General PSR parameters
C     the psr index number
      IPSR = IQFUN(1)
C     time (sec)
      TIM  = RQFUN(1)
C     temperature (K)
      TEMP = RQFUN(2)
C     pressure (ergs)
      PERG = RQFUN(3)
C     pressure (atm)
      PATM = RQFUN(4)
C     power (ergs)
      POWR = RQFUN(5)
C     volume (cm**3)
      VOL  = RQFUN(6)
C     area (cm**2)
      AREA = RQFUN(7)
C
C     Material-specific parameters
      IMAT = IQFUN(2)
C     surface temperature (K)
      TSURF = RQFUN(8)
C     bias power (ergs/s)
      BIASPW = RQFUN(9)
C     material area fraction
      AFRAC = RQFUN(10)
C
C     ICEN parameters
C     rpm
      RPM = RQFUN(11)
C     compression ratio
      CMPR = RQFUN(12)
C     ratio of length of engine connection rod to
C     crank radius
      XOVERR = RQFUN(13)
C     engine cylinder clearance volume (cm**3)
      VOLC = RQFUN(14)
C     energy conversion factor (erg/cal)
      CALERG = RQFUN(15) 
C
C     Set the heat loss as a function of the parameters.
C     The units of QFUN must be in erg/sec. 
      PSQFUN = 0.
C
C     end of FUNCTION PSQFUN
      RETURN
      END
C
C-----------------------------------------------------------------------
C
      SUBROUTINE USRINLET (LIUIN, IINWRK, LRUIN, RINWRK, INAME, KNAMES,
     1                     FLRT, TINL, TEIN, XIN)
C  This is a USER SUBROUTINE for defining Inlet properties
C  as an arbitrary function of time.
C  Use of this subroutine is controlled by the AURORA
C  keyword, USRIN, as described in the AURORA user's manual.
C
C  The subroutine USRINLET is used to supply values of
C      FLRT   - Real scalar, Mass flow rate in g/s
C               (or if SCCM are the preferred units,
C               use keyword LFPSC in addition to USRIN)
C      TINL   - Real scalar, Inlet temperature (K)
C      TEIN   - Real scalar, Inlet electron temperature (K)
C      XIN(*) - Real array, Gas-phase reactant composition of
C               the inlet (mole fraction);
C               The length of this array is *exactly* KKGAS,
C               the number of gas-phase species in the problem.
C  and the user may set the Integer flag
C   IINWRK(1) - if 0, inlet parameter setting was successful,
C               else, there was a problem, so discontinue calculations
C
C  Given the following data:
C
C      LIUIN - Integer scalar, length of some Integer workspace
C      IINWRK(*) - Integer workspace array, containing 
C      IINWRK(2), LOUT - if positive, the unit number of an open output file
C      IINWRK(3), IPSR - PSR index number
C      IINWRK(4), IINL - Index number of an Inlet of IPSR
C      IINWRK(5), KKGAS- Gas-phase species count
C      IINWRK(6), LENRGY-If 0, then inlet temperature TINL is fixed, 
C                              else TINL must be supplied here
C      IINWRK(7), LENRGE-If 0, then inlet electron temperature TEIN is fixed,
C                              else TEIN must be supplied here
C  
C      LRUIN - Integer scalar, length of some Real workspace
C      RINWRK(*) - Real workspace array, containing
C      RINWRK(1), TSTART - Initial time (sec.) of calculation
C      RINWRK(2), TIM - Current time (sec.)
C
C      INAME - Character-string Inlet name
C      KNAMES- Character-string array, Gas-phase species names
C*****EXCLUDE-IF precision_single
      IMPLICIT DOUBLE PRECISION (A-H, O-Z), INTEGER (I-N)
      PARAMETER (PATM = 1.01325D6,TSTD=273.15)
C*****END EXCLUDE-IF precision_single
C*****INCLUDE-IF precision_single
C      IMPLICIT REAL (A-H, O-Z), INTEGER (I-N)
C      PARAMETER (PATM = 1.01325E6)
C*****END INCLUDE-IF precision_single
C     Variables passed in from calling routine
      DIMENSION IINWRK(LIUIN), RINWRK(LRUIN)
      CHARACTER*16 INAME, KNAMES(*)
C     Arrays returned by this user routine
      DIMENSION XIN(*)
C     Local variables:
      CHARACTER*80 MYFILE
      PARAMETER (MYFILE = 'trans.txt',LUNIT=33,
     1           MXPTS=95,MXSPEC=3)
C     Local storage space for variables
      CHARACTER*16 MYSPEC(MXSPEC)
      CHARACTER*80 HEADER
      LOGICAL LENRGY,LENRGE
      DIMENSION PPM(MXSPEC)
      COMMON/USRINL1/ MAPSP(MXSPEC),NPTS
      COMMON/USRINL2/ TIMEPT(MXPTS),SLMPT(MXPTS),XINPT(MXPTS,MXSPEC),
     1                TPT(MXPTS)
C
C     set error flag
      IINWRK(1) = 0
C     set local variables from workspace data provided
      LOUT = IINWRK(2)
      IPSR = IINWRK(3)
      IINL = IINWRK(4)
      KKGAS= IINWRK(5)
      LENRGY=IINWRK(6).GT.0
      LENRGE=IINWRK(7).GT.0
C
      TSTART=RINWRK(1)
      TIME  =RINWRK(2)
C
C     Initialize returned variables      
      FLRT = 0.0
      TINL = 298.
      TEIN = 298.
      DO K = 1, KKGAS
         XIN(K) = 0.0
      ENDDO
C
C     First time in, read in all the points so we don't have to do 
C     IO on each call.  Interpolate from saved points thereafter      
      IF (TIME .EQ. TSTART) THEN
C        Open and read the time-date file
C        Store points in arrays for access/interpolation at later times
         IOS = 0
         OPEN(LUNIT, FILE=MYFILE, FORM='FORMATTED', STATUS='OLD', 
     1        IOSTAT=IOS)

C        Check that file open was successful, if not return with error      
         IF (IOS .NE. 0) GO TO 1000
C
C        Map input species to CHEMKIN names and find indices
         MYSPEC(1) = 'NO2'
         MYSPEC(2) = 'NO'
         MYSPEC(3) = 'N2'


         DO MYK = 1, MXSPEC
            CALL CKCOMP(MYSPEC(MYK),KNAMES,KKGAS,INDX)
            IF (INDX .GT. 0 .AND. INDX .LE. KKGAS) THEN
               MAPSP(MYK) = INDX
            ENDIF
         ENDDO
         
C        Read in the arrays of available information
C        In this case file format is as follows:
C        Time(s),T(C),Flrt(SLM), species (ppm) 
         READ(LUNIT, '(A)', END=800, ERR=1000) HEADER
         NPTS = 0
         ZERO = 0.0
         DO I = 1, MXPTS
            READ(LUNIT, *, END=800, ERR=1000) 
     1         TIMEPT(I),TCELS,SLM,(PPM(K),K=1,MXSPEC-1)
            NPTS = NPTS + 1
            TPT(I) = TCELS + 273.15
            SLMPT(I) = SLM
            XSUM = 0.0
            DO MYK = 1, MXSPEC-1
               XINPT(I,MYK) = PPM(MYK) * 1.E-6
               XSUM = XSUM + XINPT(I,MYK)
            ENDDO
C           Set fraction of inert  = 1 minus the sum of others            
            XREM = 1.0-XSUM
            XINPT(I,MXSPEC) = MAX(XREM,ZERO)
C           Note: if XSUM > 0.0, AURORA will normalize so that sum = 1            
         ENDDO
  800    CONTINUE
         IF (NPTS .EQ. 0) GO TO 1000 
         CLOSE(LUNIT)
      ENDIF
C     Interpolate data for input time and perform units conversions 
      XSUM = 0.0
      DO MYK = 1, MXSPEC-1
         XIN(MAPSP(MYK)) = CKBSEC(NPTS,TIME,TIMEPT,XINPT(1,MYK))
         XSUM = XSUM + XIN(MAPSP(MYK))
      ENDDO
      XREM = 1.0-XSUM
      XIN(MAPSP(MXSPEC)) = MAX(XREM, ZERO)
      SLM = CKBSEC(NPTS,TIME,TIMEPT,SLMPT)
C     Convert from SLM to SCCM
      FLRT = SLM * 1000.
      IF (LENRGY) THEN
C        Set the gas inlet temperature
         TINL = CKBSEC(NPTS,TIME,TIMEPT,TPT)
      ENDIF
      IF (LENRGE) THEN
C        Set the electron inlet temperature
         TEIN = TINL
      ENDIF
      RETURN
C
 1000 CONTINUE
      IF (IOS .NE. 0) THEN
         WRITE (LOUT, *) ' ERROR...OPEN failure on inlet data file'
      ELSE
         WRITE (LOUT, *) ' ERROR...READ failure on inlet data file'
         CLOSE (LUNIT)
      ENDIF
      IINWRK(1) = 1
C      
      RETURN
      END


